/* Copyright (c) 1997-2013
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.org

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: convert_group_domain.cc 11061 2012-12-29 17:25:41Z pfeifle $
*/

#include "polymake/client.h"
#include "polymake/IncidenceMatrix.h"
#include "polymake/Map.h"
#include "polymake/Set.h"
#include "polymake/Array.h"
#include "polymake/group/group_domain.h"

namespace polymake { namespace polytope {

perl::Object convert_group_domain(const perl::Object g_in, const IncidenceMatrix<>& VIF)
{
   perl::Object g_out(g_in.type());
   g_out.set_description() << "domain conversion of " << g_in.name() << " (" << g_in.description() << ")";

   const int in_domain = g_in.give("DOMAIN");
   IncidenceMatrix<> I;
   switch (in_domain) {
   case polymake::group::OnFacets:
      g_out.set_name("fullCombinatorialGroupOnRays");
      g_out.take("DOMAIN") << polymake::group::OnRays;
      I = T(VIF);
      break;
   case polymake::group::OnRays:
      g_out.set_name("fullCombinatorialGroupOnFacets");
      g_out.take("DOMAIN") << polymake::group::OnFacets;
      I = VIF;
      break;
   default:
      cerr << "The domain of the input group is " << in_domain << endl;
      throw std::runtime_error("Cannot handle this type of input domain.");
   }

   // initialize auxiliary data structures
   const int n_rows = I.rows();
   Array<Set<int> > row_sets(n_rows);
   Map<Set<int>, int> index_of;
   int ct(0);
   for (Entire<Rows<IncidenceMatrix<> > >::const_iterator rit = entire(rows(I)); !rit.at_end(); ++rit, ++ct) {
      const Set<int> row(*rit);
      index_of[row] = ct;
      row_sets[ct] = row;
   }
   
   const Array<Array<int> > gens_in = g_in.give("GENERATORS");
   Array<Array<int> > gens_out(gens_in.size());
   for (int i=0; i<gens_in.size(); ++i) {
      // for each input generator ...
      Array<int> gen_out(n_rows);
      for (int j=0; j<n_rows; ++j) {
         // for each row of the incidence matrix, regarded as an index set ...
         Set<int> image;
         for (Entire<Set<int> >::const_iterator sit = entire(row_sets[j]); !sit.at_end(); ++sit)
            // find the image of the row under the input generator
            image += gens_in[i][*sit];
         // ... and store the index of that image in a new output generator
         gen_out[j] = index_of[image];
      }
      gens_out[i] = gen_out;
   }

   g_out.take("GENERATORS") << gens_out;
   return g_out;
}

UserFunction4perl("# @category Symmetry"
                  "# converts the generators of the input group from the domain onRays "
                  "# to generators on the domain onFacets, and vice versa. "
                  "# @param group::Group group input group "
                  "# @param IncidenceMatrix VIF the vertex-facet incidence matrix of the cone or polytope"
                  " @return group::Group a new group object with the generators induced on the new domain",
                  &convert_group_domain, "convert_group_domain(group::Group, IncidenceMatrix)");

} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:

