/* Copyright (c) 1997-2013
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.org

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: cocircuit_equations.cc 11055 2012-12-20 16:41:03Z pfeifle $
*/


#include "polymake/client.h"
#include "polymake/SparseVector.h"
#include "polymake/ListMatrix.h"
#include "polymake/IncidenceMatrix.h"
#include "polymake/linalg.h"
#include "polymake/hash_map"
#include "polymake/Set.h"
#include "polymake/group/permlib.h"
#include "polymake/common/lattice_tools.h"
#include "polymake/common/boost_dynamic_bitset.h"
#include <iostream>
#include <fstream>
#include <vector>

namespace polymake { namespace polytope {

      //typedef Set<int> SetType;
typedef common::boost_dynamic_bitset SetType;

template<typename Scalar>
ListMatrix<SparseVector<int> > 
symmetrized_cocircuit_equations(int d,
                                const Matrix<Scalar>& V,
                                const IncidenceMatrix<>& VIF,
                                const Array<Array<int> >& generators,
                                const Array<SetType>& interior_ridge_reps,
                                const Array<SetType>& facet_reps,
                                perl::OptionSet options) 
{
   const std::string filename = options["filename"];
   std::ofstream outfile(filename.c_str(), std::ios_base::trunc);
   const group::PermlibGroup sym_group(generators);

   hash_map<SetType, int> index_of_facet;
   int ct(-1);
   for (Entire<Array<SetType> >::const_iterator ait = entire(facet_reps); !ait.at_end(); ++ait)
      index_of_facet[*ait] = ++ct;

   const int 
      n_facets(index_of_facet.size()),
      n(V.rows());
   ListMatrix<SparseVector<int> > cocircuit_eqs(0, n_facets);
   //   DataCache<Scalar, SetType> cache(V, VIF, sym_group);

   ct = 0;
   time_t start_time, current_time;
   time(&start_time);
   const int log_frequency = options["log_frequency"];
   for (Entire<Array<SetType> >::const_iterator rit = entire(interior_ridge_reps); !rit.at_end(); ++rit) {
      if (log_frequency && (++ct % log_frequency == 0)) {
         time(&current_time);
         cerr << ct << " " << difftime(current_time, start_time) << endl;
      }
      SparseVector<int> eq(n_facets);
      const SetType ridge_rep(*rit);
      const Vector<Scalar> vals = V * (null_space(V.minor(ridge_rep, All))[0]);
      for (int i=0; i<vals.size(); ++i) {
         const int s = sign(vals[i]);
         if (s!=0) {
            SetType facet(ridge_rep);
            facet.resize(n);
            facet += i;
            eq[index_of_facet[sym_group.lex_min_representative(facet)]] += s * sym_group.setwise_stabilizer(facet).order();
         }
      }
      if (eq.size()) {
         eq = common::divide_by_gcd(eq);
         cocircuit_eqs /= eq;
         if (filename.size()) wrap(outfile) << eq << endl;
      }
   }
   return cocircuit_eqs;
}

template<typename Scalar>
ListMatrix<SparseVector<int> >
symmetrized_foldable_cocircuit_equations(int d,
                                         const Matrix<Scalar>& V,
                                         const IncidenceMatrix<>& VIF,
                                         const Array<Array<int> >& generators,
                                         const Array<SetType>& interior_ridge_reps,
                                         const Array<SetType>& facet_reps,
                                         perl::OptionSet options) 
{
   const group::PermlibGroup sym_group(generators);

   const std::string filename = options["filename"];
   std::ofstream outfile(filename.c_str(), std::ios_base::trunc);

   hash_map<SetType, int> index_of_facet;
   int n_facet_reps(0);
   for (Entire<Array<SetType> >::const_iterator ait = entire(facet_reps); !ait.at_end(); ++ait)
      index_of_facet[*ait] = n_facet_reps++;
   
   ListMatrix<SparseVector<int> > cocircuit_eqs(0, 2*n_facet_reps);

   // use int instead of Rational to save time;
   //   we don't use row reductions (experimentally bad)
   SparseVector<int> eq_0_first, eq_1_first;

   // for each interior ridge rho and c in {0,1}:
   //   sum_{sigma > rho, orientation=+} x_{c,sigma} = sum_{sigma > rho, orientation=-} x_{1-c,sigma}
   for (Entire<Array<SetType> >::const_iterator rit = entire(interior_ridge_reps); !rit.at_end(); ++rit) {
      const Set<int> rho = *rit;
      cerr << rho << endl;
      eq_0_first = SparseVector<int>(2*n_facet_reps);
      eq_1_first = SparseVector<int>(2*n_facet_reps); 
      const SparseVector<Scalar> nv = null_space(V.minor(*rit, All)).row(0);
      int row_index(0); 
      for (typename Entire<Rows<Matrix<Scalar> > >::const_iterator vit = entire(rows(V)); !vit.at_end(); ++vit, ++row_index) {
         const int orientation = sign(nv * (*vit));
         if (orientation != 0) {
            SetType facet(*rit);
            facet.resize(V.rows());
            facet += row_index;
            const SetType this_facet(sym_group.lex_min_representative(facet));
            const int 
               iof (index_of_facet[this_facet]),
               mult(sym_group.setwise_stabilizer(this_facet).order());
            cerr << "  " << facet << " " << this_facet << " " << orientation << " (" << 2*iof << "," << 2*iof+1 << ") " << mult << endl;
            if (orientation>0) {
               eq_0_first[2*iof] +=  mult;
               eq_1_first[2*iof+1] +=  mult;
            } else {
               eq_0_first[2*iof+1] +=  -mult;
               eq_1_first[2*iof] +=  -mult;
            }
         }
      }
      cocircuit_eqs /= eq_0_first;
      cocircuit_eqs /= eq_1_first;
      if (filename.size())
         wrap(outfile) << eq_0_first << "\n" << eq_1_first << endl;
   }
   return cocircuit_eqs;     
}

FunctionTemplate4perl("symmetrized_cocircuit_equations<Scalar>($ Matrix<Scalar> IncidenceMatrix Array<Array<Int>> Array<common::boost_dynamic_bitset> Array<common::boost_dynamic_bitset> { filename=>'', reduce_rows=>0, log_frequency=>0 })");

FunctionTemplate4perl("symmetrized_foldable_cocircuit_equations<Scalar>($ Matrix<Scalar> IncidenceMatrix Array<Array<Int>> Array<common::boost_dynamic_bitset> Array<common::boost_dynamic_bitset> { filename=>'', reduce_rows=>0, log_frequency=>0 })");


} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:

 
